package com.jiuqi.syntax;

public class UsingExample implements SyntaxListener
{
    private static String[] zbcodes;
    private static double[] zbvalues;
    
    static {
        UsingExample.zbcodes = new String[10];
        UsingExample.zbvalues = new double[10];
        UsingExample.zbcodes[0] = "ZB0001";
        UsingExample.zbcodes[1] = "ZB0002";
        UsingExample.zbcodes[2] = "ZB0003";
        UsingExample.zbcodes[3] = "ZB0004";
        UsingExample.zbcodes[4] = "ZB0005";
        UsingExample.zbcodes[5] = "ZB0006";
        UsingExample.zbcodes[6] = "ZB0007";
        UsingExample.zbcodes[7] = "ZB0008";
        UsingExample.zbcodes[8] = "ZB0009";
        UsingExample.zbcodes[9] = "ZB0010";
        UsingExample.zbvalues[0] = 1.0;
        UsingExample.zbvalues[1] = 10.0;
        UsingExample.zbvalues[2] = 100.0;
        UsingExample.zbvalues[3] = 1000.0;
        UsingExample.zbvalues[4] = 10000.0;
        UsingExample.zbvalues[5] = 100000.0;
        UsingExample.zbvalues[6] = 1000000.0;
        UsingExample.zbvalues[7] = 1.0E7;
        UsingExample.zbvalues[8] = 1.0E8;
        UsingExample.zbvalues[9] = 1.0E9;
    }
    
    public int getValue(final SyntaxEvent syntaxEvent) {
        if (!syntaxEvent.dataCell.isZB) {
            return 1;
        }
        int foundIndex = -1;
        for (int i = 0; i < UsingExample.zbcodes.length; ++i) {
            if (syntaxEvent.dataCell.zbName.equalsIgnoreCase(UsingExample.zbcodes[i])) {
                foundIndex = i;
                break;
            }
        }
        if (foundIndex < 0) {
            return 1012;
        }
        syntaxEvent.dataCell.offset = foundIndex;
        syntaxEvent.value.type = 2;
        syntaxEvent.value.cdReal = UsingExample.zbvalues[foundIndex];
        return 0;
    }
    
    public int setValue(final SyntaxEvent syntaxEvent) {
        if (!syntaxEvent.dataCell.isZB) {
            return 1;
        }
        int foundIndex = -1;
        for (int i = 0; i < UsingExample.zbcodes.length; ++i) {
            if (syntaxEvent.dataCell.zbName.equalsIgnoreCase(UsingExample.zbcodes[i])) {
                foundIndex = i;
                break;
            }
        }
        if (foundIndex < 0) {
            return 1012;
        }
        syntaxEvent.dataCell.offset = foundIndex;
        UsingExample.zbvalues[foundIndex] = syntaxEvent.value.cdReal;
        return 0;
    }
    
    public void getNiValue(final SyntaxEvent syntaxEvent) {
        syntaxEvent.value.type = 2;
        syntaxEvent.value.cdReal = UsingExample.zbvalues[syntaxEvent.dataCell.offset];
    }
    
    public void setNiValue(final SyntaxEvent syntaxEvent) {
        syntaxEvent.value.type = 2;
        UsingExample.zbvalues[syntaxEvent.dataCell.offset] = syntaxEvent.value.cdReal;
    }
    
    public int getStatValue(final SyntaxEvent syntaxEvent) {
        return 1;
    }
    
    public void getCellTitle(final SyntaxCellTitleEvent syntaxCellTitleEvent) {
    }
    
    public void getWildcardAreas(final WildcardAreasEvent wildcardAreasEvent) {
    }
    
    public int getDictMeaning(final DictMeaningEvent dictMeaningEvent) {
        return 1;
    }
    
    public int getUnitState(final UnitStateEvent unitStateEvent) {
        return 1;
    }
    
    public int getFloatStat(final FloatStatEvent floatStatEvent) {
        return 1;
    }
    
    public int execFloatCopy(final FloatCopyEvent floatCopyEvent) {
        return 1;
    }
    
    public int getLinkPlus(final LinkPlusEvent linkPlusEvent) {
        return 1;
    }
    
    public void finishOneExpress() {
    }
    
    public static void main(final String[] args) {
        final SyntaxParser parser = new SyntaxParser();
        final UsingExample exampleListener = new UsingExample();
        parser.setSyntaxListener(exampleListener);
        parser.hasCellSyntax = true;
        parser.hasStarSyntax = false;
        final CommonData value = new CommonData();
        System.out.println("1、使用非Niporlan方式的调用方法");
        parser.hasMakeNiporlan = false;
        if (parser.attract("[zb0001]+[zb0004]", value) != 0) {
            System.out.println(parser.getErrorMessage());
        }
        System.out.println("[zb0001]+[zb0004]的结果是：" + value.toString());
        if (parser.evaluate("[zb0001] = [zb0002]*[zb0003]+[zb0004]") != 0) {
            System.out.println(parser.getErrorMessage());
        }
        System.out.println("[zb0001] = [zb0002]*[zb0003]+[zb0004]的计算结果：[zb0001]=" + UsingExample.zbvalues[0]);
        System.out.println("2、使用Niporlan方式的调用方法");
        parser.hasMakeNiporlan = true;
        if (parser.attract("[zb0001]+[zb0004]", value) != 0) {
            System.out.println(parser.getErrorMessage());
            return;
        }
        parser.attract(parser.getNiporlan(), value);
        System.out.println("[zb0001]+[zb0004]的结果是：" + value.toString());
        if (parser.evaluate("[zb0001] = [zb0002]*[zb0003]+[zb0004]") != 0) {
            System.out.println(parser.getErrorMessage());
            return;
        }
        parser.evaluate(parser.getNiporlan());
        System.out.println("[zb0001] = [zb0002]*[zb0003]+[zb0004]的计算结果：[zb0001]=" + UsingExample.zbvalues[0]);
        System.out.println("3、使用Niporlan方式的批量调用方法");
        parser.hasMakeNiporlan = true;
        final String[] formulas = new String[3];
        final Niporlan[] niporlans = new Niporlan[3];
        formulas[0] = "[zb0001] = [zb0002]*[zb0003]+[zb0004]";
        formulas[1] = "[zb0005] = ABS([zb0006]-[zb0007])";
        formulas[2] = "[zb0010] = [zb0008]-[zb0009]";
        for (int i = 0; i < formulas.length; ++i) {
            if (parser.evaluate(formulas[i]) != 0) {
                System.out.println(parser.getErrorMessage());
                return;
            }
            niporlans[i] = parser.getNiporlan();
        }
        for (int i = 0; i < niporlans.length; ++i) {
            parser.evaluate(niporlans[i]);
        }
        System.out.println("4、看看错误的公式是如何提示的");
        if (parser.attract("[zb0001]+[zb0004", value) != 0) {
            System.out.println(parser.getErrorMessage());
        }
        if (parser.evaluate("[zb0001] = [abc]*[zb0003]+[zb0004]") != 0) {
            System.out.println(parser.getErrorMessage());
        }
    }
    
    public void finishOneExpress(final boolean hasError) {
    }
}
